let cmp = fun x y -> if x > y then 1 else if x < y then -1 else 0;;

(* Un module dont on aimerait qu'il soit inutile *)
module Debug =
  struct
    let out_message = ref (fun x -> ())
    let out_flush = ref (fun x -> ())
    let out_int = ref (fun x -> ())
    exception Debug of string
    
    let dbg a b = 
      begin 
        !out_message a; !out_flush stdout; b 
      end

  end;;

(* Fonctions gnrales - diverses et varies *)
module Helpers =
  struct
    let is_in elt = List.exists (fun x -> x = elt)

    let rec insert e = function
      []     -> [e]
    | (a::b) -> if a = e 
                  then a::b
                  else 
                    if a < e 
                      then a::(insert e b)
                      else e::(a::b)
                      
    (* Note UC c'est pour un-curry *)
    let merge a b =
      let rec merge_uc = function
        ([], t)          -> t
      | (t, [])          -> t
      | ((a::b), (c::d)) -> if a < c 
                              then a::merge_uc (b, (c::d))
                              else 
                                if a = c 
                                  then a::merge_uc (b, d)
                                  else c::merge_uc ((a::b), d)
      in merge_uc (a, b)

    let concat a b = 
      let rec concat_uc = function
        ([], t)          -> t
      | (t, [])          -> t
      | ((a::b), (c::d)) -> if a < c 
                              then a::concat_uc (b, (c::d))
                              else c::concat_uc ((a::b), d)
        in concat_uc (a, b)
                       
    let rec sub_part = function
      []     -> [[]]
    | (a::b) -> let t = sub_part b 
                  in merge (List.map (fun x -> a::x) t) t

    let rec prive a b = match a with
      []    -> []
    |  x::y -> if is_in x b then prive y b else x::prive y b

    let flat_map a b = List.flatten (List.map a b)
    
    let rec all_assoc elt = function
      []         -> []
    |  (a, b)::c -> if a = elt 
                      then b::all_assoc elt c 
                      else all_assoc elt c

    let rec supp_doub = function
      []      -> []
    | [a]     -> [a]
    | a::b::s -> if a = b 
                   then supp_doub (b::s)
                   else a::supp_doub (b::s)
  end;;


(* Les types fondamentaux du programme. *)
module Types =
  struct
    type constante = int * int
    type label = constante list
    type variable = int
    type ('a, 'b) btree = Leaf of 'a
                        | Node of ('a, 'b) btree 
                                * 'b 
                                * ('a, 'b) btree

    (* Si une version future voulait dessiner l'arbre... *)
    type annotation = Ann_none

    type operation = OpProdA
                   | OpProdM
                   | OpSommeA
                   | OpImplA
                   | OpImplM

    type classop = Pi_alpha | Pi_beta | Single | Double | Ground;;
    type polarity = bool
    type proposition = 
            V of variable
          | ProdA of proposition * proposition
          | ProdM of proposition * proposition
          | SommeA of proposition * proposition
          | ImplM of proposition * proposition
          | ImplA of proposition * proposition;;

    type formulatree = 
      (variable * polarity, 
       operation * classop * polarity * proposition * int) 
      btree;;

    type signed_formula = bool * proposition * label

    (* Ici encore, inusit - je le laisse pour une version
        future *)
    type tableau = 
            End    of annotation * signed_formula list
          | Node_1 of annotation * signed_formula list * tableau
          | Node_2 of annotation * signed_formula list * tableau * tableau
          | Closed_branch
          | Redundant_branch
    
    (* Pour la boucle d'valuation *)
    type injonction = No_operation 
                    | Search_depth of int
                    | Check of proposition
                    | Help
		    | Stat
	            | TeXcm of string
                    | Strategy of bool
		    | Syntax_error

    let i_count = ref 0

    (* TODO *)
    exception Redundant_branch
  end;;

(* Fonctions gnrales de traitement des graphes *)
module Graphes = 
  struct
    type graph = (Types.label * (Types.label list) )list
    type bi_graph = graph * graph
    type t = bi_graph

    let followers x g = 
      let step = ref [x] and ret = ref [x] in
        begin
          while !step <> [] do
            step := List.flatten 
                (List.map 
                   (fun x -> try (List.assoc x g) with Not_found -> [])
                 !step);
            ret := (!step)@(!ret);
          done;
          !ret
        end

    (* before les prdecesseurs d'un noeud
       after les successeurs
       im_ c'est pour immdiat *)
    let before x g = followers x (snd g)
    let after x g = followers x (fst g)

    let im_before x g = try List.assoc x (snd g) with Not_found -> []
    let im_after x g = try List.assoc x (fst g) with Not_found -> []

    let rec insert_item elt = function
      []     -> [elt, []]
    | (a::b) -> if (fst a) < elt 
                  then a::(insert_item elt b)
                  else 
                    if (fst a) = elt 
                      then (a::b)
                      else (elt, [])::a::b;;

    let insert_link graph (elt1, elt2) =
      if elt1 = elt2 
        then 
          (insert_item elt2 (insert_item elt1 graph)) 
        else
          let rec func1 = function
                [] -> raise (Debug.Debug "insert_link")
              | ((a, b)::c) 
                   -> if a < elt1 
                        then (a, b)::func1 c
                        else (* Si ce n'est toi... *)
                          (a, (Helpers.insert elt2 b))::c
          in 
            func1 (insert_item elt2 (insert_item elt1 graph))

    let new_link ((g, d):bi_graph) (e, f) =
      (((insert_link g (e, f)), (insert_link d (f, e))): bi_graph)

    let new_item ((g, d):bi_graph) e =
      (((insert_item e g), (insert_item e d)): bi_graph)

  end;;

(* Traitement des labels *)
module LabelWork =
  struct
    (* Dcoupe un label autant que possible *)
    let partitions e = 
      let rec parties = function
         []   -> [[]]
      | a::b -> let p = parties b in
                   (List.map (fun x -> (a::x)) p)@p
      in let p = parties e in
        List.map2 (fun x -> fun y -> (x, y)) p (List.rev p)

    (* resoud_{n} / n \in {1, 2, 3} correspond(ent ?) 
       la rsolution de contraintes *)
    let resoud_1 (* yoz <= x *) (g : Graphes.t) (elt : Types.label) unite =
      let pred (x, y) = not ((x = []) or (y = [])) in
        let suite = Graphes.before elt g in 
          List.sort cmp 
            (if unite 
               then 
                 List.flatten (List.map partitions suite)
               else 
                 List.filter pred 
                   (List.flatten (List.map partitions suite)))

    let resoud_2 (* x <= y *) (g : Graphes.t) (elt : Types.label) unite =
      List.map (fun x -> (x, x)) (List.sort cmp (Graphes.after elt g))

    let rec decoupe = function
      ((a::b), (c::d)) -> if c < a (* on abandonne tout espoir *)
                         then raise Not_found
                           else if a = c then decoupe (b, d)
                                         else a::(decoupe (b, (c::d)))
    | (t, [])          -> t
    | ([], t)          -> raise Not_found

    let resoud_3 (g : Graphes.t) (elt : Types.label) unite =
      let ll = List.map fst (fst g) in
      List.flatten (List.map 
        (fun x -> try 
                   let d = decoupe (x, elt) 
                     in if Helpers.is_in x ll 
                       then [d, x] 
                       else []
                 with Not_found -> [])
        ll)

    let rec decoupe = function
      ((a::b), (c::d)) -> if c < a (* on abandonne tout espoir *)
                            then raise Not_found
                            else 
                              if a = c 
                                then decoupe (b, d)
                                else a::(decoupe (b, (c::d)))
    | (t, [])          -> t
    | ([], t)          -> raise Not_found

    let solve_yx y x =
(* a-t-on un y(x) si oui quelle est la partie commune
   on renvoie une liste de couples (pc, occ) o occ est
   le nombre d'occurences de x qui manquent dans y *)
      let rec interm y x prof = 
        try let a = decoupe (y, x) in
          (a, prof)::interm a x (prof + 1)
        with Not_found -> []
      in if x = [] then [y, 1] else interm y x 1

    let rec substitute (b : Types.label) ((a : Types.label), i) =
      if i = 0 
        then a
        else Helpers.concat b (substitute b (a, (i-1)))

    let permargs f a = fun b -> f b a
 
    let concat_check_red e f =
      match e with
      [(i, j)] -> if Helpers.is_in j (List.map snd f) 
        then 
          begin 
            !Debug.out_message "\nredundant branch\n";
            !Debug.out_flush stdout; 
            raise Types.Redundant_branch 
          end    
        else Helpers.concat e f
      |  _ -> Helpers.concat e f

  end;;

(* Fonctions gnrales encore *)
module ImpHelpers =
  struct
    let current_ID = ref (-1)
    let new_ID () = begin incr current_ID; !current_ID end

    let rec get_id_list = function
      (Types.Leaf _ : Types.formulatree) -> []
    | Types.Node (g, (_, _, _, a, b), d) 
                   -> (b, a)::((get_id_list g)@(get_id_list d))

    let signal_nettoie = ref false
    let index_vars = ref (-1)
    let index_cons = ref (-1)

    let init_indexes   () = 
       begin index_vars := -1; index_cons := -1; end
    let new_vars_index () = 
       begin incr index_vars; !index_vars end
    let new_cons_index () = 
       begin incr index_cons; !index_cons end

  end;;

(* Fonctions gnrales toujours *)
module BIHelpers =
  struct

    let polarite = function
      (Types.Node (_, (_, _, p, _, _), _) : Types.formulatree) 
                            -> p
    | _                     -> raise (Debug.Debug "polarite")

    (* Ich wei es, eine Enumerazion ist besser aber...*)
    let ordinal = function
     (Types.Node (_, (Types.OpImplA, _, false, _, _), _) : 
              Types.formulatree)                         -> 5
    | Types.Node (_, (Types.OpImplM, _, false, _, _), _) -> 7
    | Types.Node (_, (Types.OpProdM, _, true,  _, _), _) -> 6
    | Types.Node (_, (Types.OpImplA, _, true,  _, _), _) -> 8
    | Types.Node (_, (Types.OpProdM, _, false, _, _), _) -> 9
    | Types.Node (_, (Types.OpImplM, _, true,  _, _), _) -> 10
    | _                                    -> raise (Debug.Debug "ordinal")

    let string_of_proposition = function (p : Types.proposition) ->
      let rec sof p = match p with
	  Types.SommeA (p1,p2) -> "(" ^ sof(p1) ^ "v" ^ sof(p2) ^ ")"
	| Types.ProdA (p1,p2) -> "(" ^ sof(p1) ^ "^" ^ sof(p2) ^ ")"
        | Types.ImplA (p1,p2) -> 
	    let res = sof(p1) ^ "->" ^  sof(p2) in
	      if (res = "T->T") then "T" 
	      else "(" ^ res ^ ")"
        | Types.ProdM (p1,p2) -> "(" ^ sof(p1) ^ "*" ^ sof(p2) ^ ")"
        | Types.ImplM (p1,p2) -> "(" ^ sof(p1) ^ "-*" ^ sof(p2) ^ ")"
	| Types.V i -> Char.escaped (Char.chr i)
      in sof p

    let formulatree_of_proposition = function (p : Types.proposition ) ->
      let rec interne v = match v with (* inlgant *)
          (pol, Types.V x)             
          -> Types.Leaf (x, pol)
	| (false, Types.SommeA (p1, p2)) 
           -> ((Types.Node (interne (false, p1), 
			    (Types.OpSommeA, Types.Single,   false, snd v, 
			     ImpHelpers.new_ID ()),
			    interne (false, p2))) : Types.formulatree)
	| (true,  Types.SommeA (p1, p2)) 
          -> Types.Node (interne (true,  p1),
			 (Types.OpSommeA, Types.Double,   true,  snd v, 
			  ImpHelpers.new_ID ()),
			 interne (true,  p2))
	| (true,  Types.ProdA  (p1, p2)) 
          -> Types.Node (interne (true,  p1),
			 (Types.OpProdA,  Types.Single,   true,  snd v, 
			  ImpHelpers.new_ID ()),
			 interne (true,  p2))
	| (false, Types.ProdA  (p1, p2)) 
          -> Types.Node (interne (false, p1),
			 (Types.OpProdA,  Types.Double,   false, snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (false, p2))
	| (false, Types.ImplA  (p1, p2)) 
          -> Types.Node (interne (true,  p1),
			 (Types.OpImplA,  Types.Pi_alpha, false, snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (false, p2))
	| (true,  Types.ProdM  (p1, p2)) 
          -> Types.Node (interne (true,  p1),
           (Types.OpProdM,  Types.Pi_alpha, true,  snd v, 
            ImpHelpers.new_ID ()),
           interne (true,  p2))
	| (false, Types.ImplM  (p1, p2))
          -> Types.Node (interne (true,  p1),
			 (Types.OpImplM,  Types.Pi_alpha, false, snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (false, p2))
	| (true,  Types.ImplA  (p1, p2)) 
          -> Types.Node (interne (false, p1), 
			 (Types.OpImplA,  Types.Pi_beta,  true,  snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (true,  p2))
	| (false, Types.ProdM  (p1, p2)) 
          -> Types.Node (interne (false, p1), 
			 (Types.OpProdM,  Types.Pi_beta,  false, snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (false, p2))
	| (true,  Types.ImplM  (p1, p2)) 
          -> Types.Node (interne (false,  p1),
			 (Types.OpImplM,  Types.Pi_beta,  true,  snd v, 
			  ImpHelpers.new_ID ()), 
			 interne (true, p2))
      in interne (false, p)

(* On peut faire un peu de rcriture pour amliorer l'tat de la recherche *)

    let rec transforme = function
      Types.V x         -> Types.V x
     | Types.ImplA (a, b) -> begin 
                              match transforme b with
                                Types.ImplA (c, d) 
                                  -> Types.ImplA (Types.ProdA (a, c), d)
                              | _          
                                  -> Types.ImplA (transforme a, transforme b)
                            end
    | Types.ImplM (a, b) -> Types.ImplM (transforme a, transforme b)
    | Types.SommeA (a, b) -> Types.SommeA (transforme a, transforme b)
    | Types.ProdM (a, b)  -> Types.ProdM (transforme a, transforme b)
    | Types.ProdA (a, b)  -> Types.ProdA (transforme a, transforme b)

    type direction = Left | Right

    exception It_is_too_short_a_tree
    exception Redundant_branch
    exception Open_branch 
      of (Types.variable * Types.label) list 
       * (Types.variable * Types.label) list 
       * Graphes.t
       
    let subbranch = function
      Types.Leaf _         -> raise It_is_too_short_a_tree;
    | Types.Node (l, _, r) -> function
                                Left  -> l
                              | Right -> r

    let sons = function
      Types.Leaf _         -> []
    | Types.Node (l, _, r) -> [l; r]

    let calc_id = function
      (Types.Node (_, (_, _, _, _, i), _) : Types.formulatree) -> i
    | (Types.Leaf (v, _))                                -> -v
 
  end;;
 
(* Traitement des connections. Qui l'et cru ? *)
module Connections =
  struct
    exception Found

    let recherche_connections (pos: (Types.variable * Types.label) list)
                              (neg: (Types.variable * Types.label) list)
                              (graph : Graphes.t) = 
      let rec paires_possibles pos = function
        []        -> false
      | (a, b)::q -> try 
                       begin
                         let ss = ref (Helpers.all_assoc a pos) in 
                         while !ss <> [] do
                           let s = List.hd !ss in
                             if Helpers.is_in s (Graphes.before b graph) 
                               then raise Found;
                           ss := List.tl !ss;
                         done;
                         paires_possibles pos q
                       end
                     with Found -> true
        in paires_possibles pos neg


    let rec paire_possible (a, b) (pos : (Types.variable * Types.label) list)
                              (neg : (Types.variable * Types.label) list)
                              (graph : Graphes.t) pol =
      try
        if pol then 
          let ss = ref (Helpers.all_assoc a neg) in 
             begin
                 while !ss <> [] do
                   let s = List.hd !ss in
                     if Helpers.is_in s (Graphes.after b graph) 
                       then raise Found;
                   ss := List.tl !ss;
                 done;
                 false
              end
         else
          let ss = ref (Helpers.all_assoc a pos) in 
            begin
              while !ss <> [] do
                let s = List.hd !ss in
                  if Helpers.is_in s (Graphes.before b graph) 
                    then raise Found;
                ss := List.tl !ss;
              done;
              false
            end
      with Found -> true
                 
  end;;

(* Ce module gre les diverses stratgies mises en oeuvre *)
module Choice =
  struct
    (* Le hd / tl c'est pas super *)
    let random_item l =
      let rec interm i g = function
            [] -> raise (Debug.Debug "random_item")
      | (a::b) -> if i = 0 
                    then (a, (List.rev g)@b)
                    else interm (i - 1) (a::g) b
      in 
        let n = List.length l in
          if n <= 1 
            then interm (Random.int n) [] l
            else interm (Random.int (n-1)) [] l

    let equiv a b = (snd a) = (snd b)

    (* Vive call_cc ? Les continuations c'est plus lgant
       que les exceptions *)
    exception Vive_call_cc of 
      (Types.formulatree * Types.label *
       ((Types.label * Types.label) list * 
        (Types.label * Types.label) list)) list *
        (Types.variable * Types.polarity, Types.operation * 
         Types.classop * Types.polarity * Types.proposition * int)
        Types.btree * Types.label * Types.label * Types.label * bool

    (* Pour filtrer les fils d'une formule *)
    let category_gauche gr cat a labelg 
                        (pibt : (Types.formulatree * Types.label) list) =
      begin
        match a with
           ((Types.Node (_, (_, c, _, _, id), _) : Types.formulatree) as aa)
                  when (c = cat)
                     && (cat = Types.Pi_beta)
               -> if Helpers.is_in (id, labelg) 
                       (List.map 
                         (fun (x,y) -> (BIHelpers.calc_id x, y)) pibt) 
                    then [] 
                    else
                      begin ImpHelpers.signal_nettoie := true; 
                        match BIHelpers.ordinal aa with
                          8  -> [a, labelg, (([] : (Types.label * 
                                                    Types.label) list), 
                                   LabelWork.resoud_2 gr labelg false)]
                        | 9  -> [a, labelg, ([], 
                                   LabelWork.resoud_1 gr labelg false)]
                        | 10 -> [a, labelg, ([], []) ]
                        | _  -> failwith "unexpected error"
                      end
        | Types.Node (_, (_, c, _, _, _), _) 
                    when c = cat -> [a, labelg, ([], [])]
        | Types.Leaf _           -> if cat = Types.Ground 
                                      then [a, labelg, ([], [])] 
                                     else []
        | _                      -> []
      end

    let category_droite = category_gauche

    let category gr cat a b lg ld p =
      (category_gauche gr cat a lg p)@
       (category_droite gr cat b ld p)

    (* Pas trs utile et je ne m'tends pas dessus. *)
    let cat_mod_pi_a_g gr cat a b lg ld p = 
           category_droite gr cat b ld p
  
    let recompose_signed_formula (pos: Types.formulatree) (lab: Types.label) =
      match pos with
        Types.Leaf (v, p)                   -> (p, Types.V v, lab)
      | Types.Node (_, (_, _, p, pr, _), _) -> (p, pr, lab)

(* Cette fonctions "fait un choix" i.e. regarde si une
   possibilite permet de clore directement une des deux branches
   d'un coup ce qui permettra d'aller plus vite et d'eviter des
   bifurcations trop nombreuses *)
(* renvoie (List.hd choix, List.tl choix) s'il n'y a rien a faire,
   sinon renvoie (a, b) tel que a est le choix a faire et b union a = choix *)

    let make_a_choice_to_left 
      gr a (choix : (Types.label * Types.label) list) pos neg =
        match a with
          Types.Leaf (v, p) -> 
            let rec func deb fin = 
              match deb with
                []     -> raise Not_found
              | (t::q) -> if Connections.paire_possible 
                                (v, fst t) pos neg gr p
                            then (t, (List.rev fin)@q)
                            else func q (t::fin)
            in
              func choix []
        | _ -> raise Not_found

   let make_a_choice_to_right
      gr a (choix : (Types.label * Types.label) list) pos neg =
        match a with
          Types.Leaf (v, p) -> 
            let rec func deb fin = 
              match deb with
              []     -> raise Not_found
            | (t::q) -> if Connections.paire_possible 
                              (v, snd t) pos neg gr p
                          then (t, (List.rev fin)@q)
                          else func q (t::fin)
            in
              func choix []
        | _ -> raise Not_found

    let est_max elt gr = (* Oui, bon, pardon *)
      if Graphes.im_after elt gr = [] 
        then true 
        else false

    let rec intersect a = function
      []   -> []
    | t::q -> if Helpers.is_in t a 
                then t::intersect a q 
                else intersect a q
  
    let rec un_max l gr = 
      let ll = intersect l (
                  Helpers.flat_map (fun x -> Graphes.im_after x gr) l) in
        if ll = [] 
          then List.hd l
          else un_max ll gr
  
(* Le dernier element vient d'etre traite on ne le reprend pas de suite *)
    let make_a_choice gr a b choix pos neg i lab =
      try make_a_choice_to_left gr a choix pos neg
      with Not_found ->
        try make_a_choice_to_right gr b choix pos neg
        with Not_found -> 
          if i = 8 then 
                  if Helpers.is_in lab (List.map fst choix)
                    then ((lab, lab), Helpers.prive choix [lab, lab])
                  else
                    let m = un_max (List.map fst choix) gr in
                              ((m, m), Helpers.prive choix [m, m])
                       (* On recherche un lment maximal *)
                      else (random_item choix);;

  end;;

Random.self_init ();;

module RulesAlpha =
  struct
    (*  Le nom de la premire, tait Olympia 
          Les Contes d'Hoffman, Acte I (Introduction) *)
    let insert_rule_FImplA (graph: Graphes.t) (c1: Types.label) 
                                              (lab: Types.label) = 
      let a = ref (Helpers.supp_doub 
                     (Helpers.flat_map (fun x -> LabelWork.solve_yx x lab)
                       (List.map fst (fst graph))))
      and b = ref graph
      in
        begin
          while !a <> [] do
            let sc = List.hd !a in let lci = LabelWork.substitute c1 sc in
            let ydx = LabelWork.substitute lab sc in
              b := Graphes.new_link !b (ydx, lci);
            let c = ref (Helpers.flat_map 
                           (LabelWork.permargs LabelWork.solve_yx lab) 
                           (Graphes.im_after ydx graph)) in (* avant - apres *)
              while !c <> [] do
                b := Graphes.new_link !b (lci, 
                                  (LabelWork.substitute c1 (List.hd !c)));
                c := List.tl !c;
              done;
            a := List.tl !a;
          done;
          !b
        end

(*  Antonia ? Morte !  
     Les Contes d'Hoffman, Acte V (Epilogue) *)
    let insert_rule_TMultM (graph: Graphes.bi_graph) 
                           (c1: Types.label) (c2: Types.label) 
                           (lab: Types.label) =
      let a = ref (Helpers.supp_doub 
                    (Helpers.flat_map (LabelWork.permargs 
                                       LabelWork.solve_yx lab)
                                (List.map fst (fst graph))))
      and b = ref graph
        in
          begin
            while !a <> [] do
              let hda  = List.hd !a
              and cij  = Helpers.merge c1 c2 in
              let yci  = LabelWork.substitute c1 hda
              and ycj  = LabelWork.substitute c2 hda
              and ycij = LabelWork.substitute cij hda
              and yx   = LabelWork.substitute lab hda
              in
                a := List.tl !a;
              b := Graphes.new_item (Graphes.new_item !b yci) ycj;
              b := Graphes.new_link !b (ycij, yx);
              let c = ref (Helpers.flat_map 
                             (LabelWork.permargs LabelWork.solve_yx lab) 
                             (Graphes.im_after yx graph)) in
                while !c <> [] do
                  let hdc  = List.hd !c in
                  let zci  = LabelWork.substitute c1 hdc
                  and zcj  = LabelWork.substitute c2 hdc
                  and zcij = LabelWork.substitute cij hdc
                  in
                    c := List.tl !c;
                    b := Graphes.new_link 
                           (Graphes.new_link
                             (Graphes.new_link !b (yci, zci))
                               (ycj, zcj))
                                 (ycij, zcij);
                done;
            done;
            !b
          end
      
(*  Giulietta palsambleu sinon, sinon, plus de fte ! 
       Les Contes d'Hoffman, Acte IV (Giulietta) *)
    let insert_rule_FImplM (graph: Graphes.t) (c1: Types.label) 
                                              (lab: Types.label) =
      let aa = Helpers.merge 
                (Graphes.before lab graph) 
                (Graphes.after lab graph) in
      let a = ref (Helpers.supp_doub (Helpers.flat_map 
                                      Helpers.sub_part aa)) in
      let b = ref graph in
        begin
          while !a <> [] do
            let y = List.hd !a in
            let evz = ref (Graphes.im_after y graph) in (* avant - apres *)
              begin
                b := Graphes.new_item !b (LabelWork.concat_check_red y c1);
                while !evz <> [] do
                  b := Graphes.new_link !b 
                      (LabelWork.concat_check_red c1 y, 
                       LabelWork.concat_check_red c1 (List.hd !evz));
                  evz := List.tl !evz;
                done;
              end;
            a := List.tl !a;
          done;
          !b
        end

  end;;

let formule = ref (Types.V 1);;
  
module Tableau = 
  struct
    (* Une fonction qui nettoie la liste des pi_b a traiter
       Si on a T A->B : x et T A->B : y et x <= y alors
       le second ne sert a rien - mais tout ce qui a t trait est utile *)
   
    let nettoie x gr = 
      let rec net = function
      ([], _)            
              -> []
    | (((a, l, _) as t)::q, dt) 
              -> if (BIHelpers.ordinal a) = 8 
                   then
                     if List.exists 
                        (fun (b, ll, _) -> ((BIHelpers.calc_id a) = 
                                            (BIHelpers.calc_id b))
                           && (Helpers.is_in l (
                                Graphes.after ll gr))) (q@dt)
                       then net (q, dt)
                       else t::net (q, t::dt)
                   else t::net (q, t::dt)
    in 
      if !ImpHelpers.signal_nettoie 
        then 
          begin
            (*  signal_nettoie := false;*)
            net (x, []) 
          end
        else x

    (* Passons [enfin]  la fonction principale *)
    (*  Pars pour Cythre !  La Belle Hlne, acte I *)
    (*  Faut pas faut pas faire un faux pas  R. Devos *)

    let compteur = ref 0
    let bi_compteur = ref 0

    let rec immerge_beta ((p, l, (tr, atr)) as pib) = function
      []     -> [pib]
    | ((a, l2, (tr2, atr2))::b) 
             -> if ((BIHelpers.calc_id a) = 
                    (BIHelpers.calc_id p)) && (l = l2) 
                  then (p, l, (Helpers.merge tr tr2, atr2))::b
                  else (a, l2, (tr2, atr2))::immerge_beta pib b

    let rec merge_beta a b = match a with
      []   -> b
    | t::q -> merge_beta q (immerge_beta t b)

    exception Continue

    (* Aucun commentaire pour cette fonction - cf histoire.tex et
       la fonction suivante *)
    let rec step_tableau_single prof
      (pos: (Types.variable * Types.label) list)
      (neg: (Types.variable * Types.label) list)
      (graphe: Graphes.t) 
      (gr: (Types.formulatree * Types.label * ('a * 'b)) list)
      (simp: (Types.formulatree * Types.label * ('a * 'b)) list)
      pi_a doub pi_b
      (pibt: (Types.formulatree * Types.label) list) =
    begin
     if prof <= 0 
       then ()
       else
        let pi_bb = pibt in
          if gr != [] then
            match List.hd gr with
              ((Types.Leaf (var, p) as a), lab ,_) ->
                let (ppos, nneg) = 
                  if p 
                    then (Helpers.insert (var, lab) pos, neg)
                    else (pos, Helpers.insert (var, lab) neg)
                  in 
                    if Connections.recherche_connections ppos nneg graphe 
                    then 
                      begin
                        raise Connections.Found;
                      end
                    else
                      step_tableau_single (prof - 1)
                                          ppos nneg graphe 
                                          (List.tl gr) simp pi_a doub pi_b pibt
            | _ -> failwith "unexpected error"
          else 
            if simp != [] 
              then
                match List.hd simp with
                  (((Types.Node (g, (_, _, p, _, id), d)) as a), 
                        lab, _) -> 
                      let (ppos, nneg) = 
                        if p 
                          then (Helpers.insert (-id, lab) pos, neg)
                          else (pos, Helpers.insert (-id, lab) neg) 
                      in
                        (step_tableau_single (prof - 1) ppos nneg graphe
                          (Choice.category graphe Types.Ground   
                             g d lab lab pi_bb)
                         ((Choice.category graphe Types.Single   
                             g d lab lab pi_bb)@
                             (List.tl simp))
                         ((Choice.category graphe Types.Pi_alpha 
                             g d lab lab pi_bb)@
                            pi_a)
                         ((Choice.category graphe Types.Double   
                             g d lab lab pi_bb)@
                            doub)
                         (pi_b)
                         pibt)
                | _ -> failwith "unexpected error"
              else if pi_a != [] 
              then 
                begin (); end(*raise Not_found*)
              else if doub != [] 
              then (* Un cas simple *)
                match List.hd doub with
                  (Types.Leaf _,_,_) -> failwith "unexpected error"
                | ((Types.Node (g, (_, _, p, _, id), d)) as a, lab, _) 
                  ->
                    let (ppos, nneg) = 
                      if p 
                        then (Helpers.insert (-id, lab) pos, neg)
                        else (pos, Helpers.insert (-id, lab) neg) 
                    in
                      begin
                        try
                          step_tableau_single (prof - 1) ppos nneg graphe
                            (Choice.category_gauche graphe Types.Ground 
                                g lab pi_bb)
                            (Choice.category_gauche graphe Types.Single
                                g lab pi_bb)
                            (Choice.category_gauche graphe Types.Pi_alpha
                                g lab pi_bb)
                           ((Choice.category_gauche graphe Types.Double
                               g lab pi_bb)
                                @(List.tl doub))
                           ((Choice.category_gauche graphe Types.Pi_beta
                                g lab pi_bb)
                                 @pi_b)
                             pibt
                        with Connections.Found ->
                          step_tableau_single (prof - 1) ppos nneg graphe
                            (Choice.category_droite graphe Types.Ground
                                d lab pi_bb)
                            (Choice.category_droite graphe Types.Single
                                d lab pi_bb)
                            (Choice.category_droite graphe Types.Pi_alpha
                                d lab pi_bb)
                           ((Choice.category_droite graphe Types.Double
                                d lab pi_bb)
                                  @(List.tl doub))
                           ((Choice.category_droite graphe Types.Pi_beta
                                d lab pi_bb)
                                  @pi_b)
                               pibt
                      end
              else if pi_b != [] 
                then (* Cette version -> recherche exhaustive *)
                  let av = ref []
                  and su = ref pi_b in
                    begin
                      while !su <> [] do
                        match List.hd !su with
                          (Types.Leaf _, _, _) -> failwith "unexpected error"
                        | (((Types.Node (g, _, d)) as a, 
                          lab, (tr, atr)) as hdsu) ->
                          begin
                            su := List.tl !su;
                            let resg = 
                              match BIHelpers.ordinal a with
                                8  -> LabelWork.resoud_2 graphe lab true
                              | 9  -> LabelWork.resoud_1 graphe lab true
                              | 10 -> LabelWork.resoud_3 graphe lab true
                              | _  -> failwith "unexpected error"
                            in 
                              let reste = ref (Helpers.prive resg tr) in
                                while !reste <> [] do
                                  let (debut, fin) = 
                                    (List.hd !reste, List.tl !reste) in
                                    match ((!av)@(!su)),a,
                                            lab, fst debut, snd debut with
                                      (_, Types.Leaf _, _,_,_) ->
                                               failwith "unexpected error"
                                    | (new_pi_b, 
                                       ((Types.Node (g, 
                                           (_, classop, pol, oper, id), d))
                                         as a), _, labg, labd)
              -> (* pardon pour l'indentation mais pour des raisons
                    d'impression je me tiens  80 colonnes de texte *)
               let pibtt = Helpers.insert (a, lab) pibt in
                  let (ppos, nneg) = 
                    if pol 
                      then (Helpers.insert (-id, lab) pos, neg)
                      else (pos, Helpers.insert (-id, lab) neg) in
                        begin
                          try 
                            step_tableau_single (prof - 1) ppos nneg graphe
                              (Choice.category_gauche graphe Types.Ground
                                   g labg pibtt)
                              (Choice.category_gauche graphe Types.Single
                                   g labg pibtt)
                              (Choice.category_gauche graphe Types.Pi_alpha
                                   g labg pibtt)
                              (Choice.category_gauche graphe Types.Double
                                   g labg pibtt)
                             ((Choice.category_gauche graphe Types.Pi_beta
                                  g labg pibtt)
                                    @new_pi_b) (pibtt);
                          with Not_found -> ();
                            |  Connections.Found -> 
                            begin
                              step_tableau_single (prof - 1) ppos nneg graphe
                                (Choice.category_droite graphe Types.Ground
                                     d labd pibtt)
                                (Choice.category_droite graphe Types.Single
                                     d labd pibtt)
                                (Choice.category_droite graphe Types.Pi_alpha
                                     d labd pibtt)
                                (Choice.category_droite graphe Types.Double
                                     d labd pibtt)
                               ((Choice.category_droite graphe Types.Pi_beta
                                     d labd pibtt)
                                   @new_pi_b) (pibtt);
                           end;
                        end;
                   (* Retour  l'indentation d'au dessus *)
                                  reste := List.tl !reste;
                                done;
                            av := hdsu::!av;
                          end;
                      done
        end
    end


    (* 0 -> pas de tests exhaustifs
       sinon mettre une valeur >0 *)
    (* let profondeur = 2 *)

    let count_call = ref 0

    let need_a_try = ref 0

    let rec step_tableau mp prof
      (pos: (Types.variable * Types.label) list)
      (neg: (Types.variable * Types.label) list)
      (graphe: Graphes.t) 
      (gr: (Types.formulatree * Types.label * ('a * 'b)) list)
      (simp: (Types.formulatree * Types.label * ('a * 'b)) list)
      pi_a doub pi_b_av
      (pibt: (Types.formulatree * Types.label) list) 
        =
        let pi_b = nettoie pi_b_av graphe in
          if Connections.recherche_connections pos neg graphe 
            then begin !Debug.out_message "closed_branch\n"; end
            else
              begin
               !Debug.out_message "current depth : ";
               !Debug.out_int prof;
               incr bi_compteur;
               if !compteur < prof 
                 then compteur := prof;
               !Debug.out_message " - max depth : ";
               !Debug.out_int !compteur;
               !Debug.out_message " - step_tableau calls : ";
               !Debug.out_int !bi_compteur;
               !Debug.out_message "\n";
               !Debug.out_flush stdout;
	       incr count_call;
               try 
                 begin
                   if gr = [] & simp = [] & doub = [] 
                     then
                       if !need_a_try = 0 
                         then
                           begin
                             need_a_try := 1;
                             step_tableau_single 
                               mp pos neg graphe gr 
                               simp pi_a doub pi_b pibt;
                           end
                         else
                           need_a_try := (!need_a_try + 1) mod 3;
                   raise Not_found;
                 end
               with
                 Connections.Found -> ();
               | Not_found -> 
    let pi_bb = pibt in
      if gr != [] then
      (* Cas 1 / on a des affirmations sur les variables non
         encore traites *)
        match List.hd gr with
          (Types.Node _, _, _) -> failwith "unexpected error"
        | ((Types.Leaf (var, p) as a), lab ,_) ->
            let (ppos, nneg) = 
              if p 
                then (Helpers.insert (var, lab) pos, neg)
                else (pos, Helpers.insert (var, lab) neg)
            in 
              if Connections.recherche_connections ppos nneg graphe 
                then begin !Debug.out_message "closed_branch\n"; end
              else
                step_tableau mp (prof + 1)
                  ppos nneg graphe 
                  (List.tl gr) simp pi_a doub pi_b pibt
      else if simp != [] then
        match List.hd simp with
          (Types.Leaf _, _, _) -> failwith "unexpected error"
        | (((Types.Node (g, (_, _, p, _, id), d)) as a), lab, _) ->
            let (ppos, nneg) = 
                    if p 
                      then (Helpers.insert (-id, lab) pos, neg)
                      else (pos, Helpers.insert (-id, lab) neg)
            in
              (step_tableau mp (prof + 1) ppos nneg graphe
                  (Choice.category graphe Types.Ground
                       g d lab lab pibt)
                 ((Choice.category graphe Types.Single
                       g d lab lab pibt)@(List.tl simp))
                 ((Choice.category graphe Types.Pi_alpha
                       g d lab lab pibt)@pi_a)
                 ((Choice.category graphe Types.Double
                       g d lab lab pibt)@doub)
                 (merge_beta (Choice.category graphe Types.Pi_beta
                               g d lab lab pibt) pi_b)
               pibt);
      else if pi_a != [] then (* Que la sarabande commence ! *)
        match List.hd pi_a with
          (Types.Leaf _, _, _) -> failwith "unexpected error"
        | ((Types.Node (g, (_, classop, pol, oper, id), d)) as a, lab, _) -> 
            let (ppos, nneg) = 
                   if pol 
                     then (Helpers.insert (-id, lab) pos, neg)
                     else (pos, Helpers.insert (-id, lab) neg) in
            let c1 = (ImpHelpers.new_cons_index (), id) in
            let (new_graph, labg, labd, cat) =
              match BIHelpers.ordinal a with
                n when n < 5 or n > 7
                  -> failwith "unexpected error"
              | 5 -> let prec = Graphes.before lab graphe in
                       if List.exists 
                             (fun t -> Helpers.is_in t prec) 
                             (Helpers.all_assoc (-(BIHelpers.calc_id g)) pos)
                         then (graphe, lab, lab, Choice.cat_mod_pi_a_g)
                         else (RulesAlpha.insert_rule_FImplA  graphe  [c1] lab, 
                                 [c1], [c1], Choice.category)
              | 6 -> (RulesAlpha.insert_rule_TMultM 
                        graphe [(-1, id)] [(-2, id)] lab, 
                        [(-1, id)], [(-2, id)], Choice.category)
              | 7 -> ((RulesAlpha.insert_rule_FImplM graphe [(-1, id)] lab), 
                        [(-1, id)], Helpers.insert (-1, id) lab, 
                        Choice.category) 
              | _ -> failwith "unexpected error"
                           (* faire index-- *) in
                (let new_pi_b = pi_b
                  in
                    step_tableau mp (prof + 1) ppos nneg new_graph
                       (cat new_graph Types.Ground
                            g d labg labd pibt)
                       (cat new_graph Types.Single
                            g d labg labd pibt)
                       ((cat new_graph Types.Pi_alpha
                            g d labg labd pibt)@
                            (List.tl pi_a))
                       ((cat new_graph Types.Double
                            g d labg labd pibt)@doub)
                       (merge_beta (cat new_graph Types.Pi_beta
                            g d labg labd pibt) 
                       pi_b) [];)
      else if doub != [] then (* Un cas simple *)
        match List.hd doub with
          (Types.Leaf _, _, _) -> failwith "unexpected error"
        | ((Types.Node (g, (_, _, p, _, id), d)) as a, lab, _) ->
          let (ppos, nneg) = 
                    if p 
                      then (Helpers.insert (-id, lab) pos, neg)
                      else (pos, Helpers.insert (-id, lab) neg) in
            begin
              step_tableau mp (prof + 1) ppos nneg graphe
                  (Choice.category_gauche graphe Types.Ground
                      g lab pibt)
                  (Choice.category_gauche graphe Types.Single
                      g lab pibt)
                  (Choice.category_gauche graphe Types.Pi_alpha
                      g lab pi_bb)
                 ((Choice.category_gauche graphe Types.Double
                      g lab pibt)@(List.tl doub))
                 (merge_beta (Choice.category_gauche graphe Types.Pi_beta
                      g lab pibt) pi_b) 
                 pibt;
              step_tableau mp (prof + 1) ppos nneg graphe
                  (Choice.category_droite graphe Types.Ground
                      d lab pibt)
                  (Choice.category_droite graphe Types.Single
                      d lab pibt)
                  (Choice.category_droite graphe Types.Pi_alpha
                      d lab pibt)
                 ((Choice.category_droite graphe Types.Double
                      d lab pibt)@(List.tl doub))
                 (merge_beta (Choice.category_droite graphe Types.Pi_beta
                      d lab pibt) pi_b)
                   pibt;
            end
      else if pi_b != [] then (* 
       Agamemnon  Mnlas "Sautez vite sautez a presse
                            Regardez l'tat de la Grce"
                         La Belle Hlne, Acte III *)
    (* Faire attention : on laisse les pi_beta traites completement
       et cela oblige  un parcours de la list *)
       try 
         let av = ref []
         and su = ref pi_b in
           begin
             while !su <> [] do
               match List.hd !su with
                 (Types.Leaf _, _, _) -> failwith "unexpected error"
               | (((Types.Node (g, _, d)) as a, lab, (tr, atr1)) as hdsu) ->
                 begin
                   su := List.tl !su;
                   let atr = Helpers.prive atr1 tr in
                     if atr != [] 
                       then
                         begin
                           let (debut, fin) 
                              = Choice.make_a_choice graphe g d atr pos neg
                                (BIHelpers.ordinal a) lab in
                             raise (Choice.Vive_call_cc
                              (
                               ((!av)@(!su)@[a, lab, 
                                 (Helpers.insert debut tr, fin)])
                             , a, lab, fst debut, snd debut, (tr = [])));
                         end
                       else (* Tout  n'est pas perdu *)
                         let resg = 
                           match BIHelpers.ordinal a with
                             8  -> LabelWork.resoud_2 graphe lab true
                           | 9  -> LabelWork.resoud_1 graphe lab true
                           | 10 -> LabelWork.resoud_3 graphe lab true
                           | _  -> failwith "unexpected error"
                           in let reste = Helpers.prive resg tr in
                             if reste <> [] then
                               begin 
                                 let (debut, fin) = 
                                    Choice.make_a_choice graphe g d 
                                       reste pos neg (BIHelpers.ordinal a) lab
                                 in
                                   raise (Choice.Vive_call_cc
                                   (
                                     ((!av)@(!su)@
                                       [a, lab, 
                                         (Helpers.insert debut tr, fin)]),
                                     a, lab, fst debut, snd debut, 
                                     (tr = [])));
                               end
                 end;
                 av := hdsu::!av;
             done; 
             raise (BIHelpers.Open_branch (pos, neg, graphe));
           end
       with (* exception => on peut encore traiter une pi_beta *)
         Choice.Vive_call_cc (new_pi_b, 
                ((Types.Node (g, (_, classop, pol, oper, id), d)) as a),
                lab, labg, labd, r_needed)
            -> let pibtt = Helpers.insert (a, lab) pibt in
               let (ppos, nneg) = 
                      if pol 
                        then (Helpers.insert (-id, lab) pos, neg)
                        else (pos, Helpers.insert (-id, lab) neg) in
                 begin
                   step_tableau mp (prof + 1) ppos nneg graphe
                     (Choice.category_gauche graphe Types.Ground
                          g labg pibtt)
                     (Choice.category_gauche graphe Types.Single
                          g labg pibtt)
                     (Choice.category_gauche graphe Types.Pi_alpha
                          g labg pibtt)
                     (Choice.category_gauche graphe Types.Double
                          g labg pibtt)
                     (merge_beta (Choice.category_gauche graphe Types.Pi_beta
                          g labg pibtt)
                       new_pi_b) (pibtt);
                   if ((BIHelpers.ordinal a) = 8) && r_needed 
                     then ()
                     else
                       step_tableau mp (prof + 1) ppos nneg graphe
                         (Choice.category_droite graphe Types.Ground
                              d labd pibtt)
                         (Choice.category_droite graphe Types.Single
                              d labd pibtt)
                         (Choice.category_droite graphe Types.Pi_alpha
                              d labd pibtt)
                         (Choice.category_droite graphe Types.Double
                              d labd pibtt)
                         (merge_beta (Choice.category_droite 
                                       graphe Types.Pi_beta  
                                       d labd pibtt) new_pi_b) (pibtt);
                 end
      else raise (BIHelpers.Open_branch (pos, neg, graphe))
    end

  let assoc_id_formula = ref []

  let make_tableau mp phi =
    begin
      compteur := 0;
      bi_compteur := 0;
      count_call := 0;
      ImpHelpers.init_indexes ();
      need_a_try := 0;
      formule := phi;
      let arbre = BIHelpers.formulatree_of_proposition 
                 (BIHelpers.transforme phi) in
      let (gro, sim, pia, dou, pib) =
        begin
          assoc_id_formula := ImpHelpers.get_id_list arbre;
          match arbre with
            (Types.Leaf (_, _)) -> ([arbre, [], ([], [])], [], [], [], [])
          | (Types.Node (_, (_, Types.Single  , _, _, _), _))
                          -> ([], [arbre, [], ([], [])], [], [], [])
          | (Types.Node (_, (_, Types.Pi_alpha, _, _, _), _))
                          -> ([], [], [arbre, [], ([], [])], [], [])
          | (Types.Node (_, (_, Types.Double  , _, _, _), _))
                          -> ([], [], [], [arbre, [], ([], [])], [])
          | (Types.Node (_, (_, Types.Pi_beta , _, _, _), _))
                          -> ([], [], [], [], [arbre, [], ([], [])])
          | _             -> failwith "unexpected error"
        end
      in step_tableau mp 0 [] [] ([[], []], [[], []])
           gro sim pia dou pib [] ;
    end
 
  end;;

(* Menelas   :  Couronnez-le, ma dame
   Helene    :  Ah, de toute mon ame !
   Le Choeur :  Gloire au berger (...)
                Il est vraiment ingnieux
        -- La Belle Helene, Acte I *)

(* Maintenant, la recherche de contre-modeles *)
(* Revisitons l'exception Open_branch *)
(* Question A - De quoi a-t-on vraiment besoin ? *)
(* Le CM est ecrit dans un fichier TeX auxiliaire, cela est effectu dans run.ml
   qui est le "toplevel" *)

(* Tout d'abord il faut renommer les constantes pour leur donner des
   noms plus agrables que c_{41}^{71}. D'autre part on peut utiliser
   les rgles d'associativit pour allger l'affichage des formules. *)
   
module TexGen =
  struct
    let constantes = ref ([]: ((int * int) * string) list)
    let index = ref 0
       
    let tex_of_constant (a, b) =
      "c_{" ^
        begin
          try List.assoc (a, b) !constantes
          with Not_found ->
            begin
              incr index;
              constantes := ((a,b), (string_of_int (!index-1)))::!constantes;
              snd (List.hd !constantes)
            end
        end ^ "}"

    let rec tex_of_label = function
      []   -> "1"
    | [a]  -> tex_of_constant a
    | t::q -> (tex_of_constant t) ^ "\\cdot " ^ (tex_of_label q)

    let rec tex_of_label_list = function
      []   -> ""
    | [a]  -> tex_of_label a
    | t::q -> (tex_of_label t) ^ "," ^ (tex_of_label_list q)

    let tex_of_graph_branch elt = function
      []   -> "\\mathcal{S}(" ^ (tex_of_label elt) ^ ")=\\emptyset \\\\ \n"
    | l    -> "\\mathcal{S}(" ^ (tex_of_label elt) 
                                ^ ")=\\left\\{" 
                                ^ (tex_of_label_list l) 
                                ^ "\\right\\} \\\\ \n"

    let rec tex_of_graph_branch_list = function
      []        -> ""
    | (a, b)::q -> (tex_of_graph_branch a b) ^ (tex_of_graph_branch_list q)


    let rec tex_of_formula = function
        Types.V i -> "\\vp{" ^ Char.escaped (Char.chr i) ^ "}"
        | Types.ImplM (a, b) -> 
	    "(" ^ (tex_of_formula a) ^ "\\mimp " ^ (tex_of_formula b) ^ ")"
	| Types.ImplA (a, b) ->
	    "(" ^ (tex_of_formula a) ^ "\\aimp " ^ (tex_of_formula b) ^ ")"
	| Types.ProdM (a, b) ->
	    "(" ^ (tex_of_formula a) ^ "\\mand " ^ (tex_of_formula b) ^ ")"
	| Types.ProdA (a, b) ->
	    "(" ^ (tex_of_formula a) ^ "\\aand " ^ (tex_of_formula b) ^ ")"
	| Types.SommeA (a, b) ->
	    "(" ^ (tex_of_formula a) ^ "\\aor " ^ (tex_of_formula b) ^ ")"


    let tex_of_id i = tex_of_formula (List.assoc i !Tableau.assoc_id_formula)

    let tex_of_models a b = if a > 0 
      then
	(tex_of_label b) ^ "\\models " ^ 
	(Char.escaped (Char.chr (a))) ^ " \\\\ \n"
      else 
	(tex_of_label b) ^ "\\models " ^ 
	(tex_of_id (-a)) ^ " \\\\ \n"

    (* On donne les affirmations positives ET les ngatives *)

    let tex_of_models_not a b = 
      if a > 0 
        then
          (tex_of_label b) 
               ^ "\\not\\models " ^ (Char.escaped (Char.chr (a))) ^ "\\\\\n"
        else 
          (tex_of_label b) 
               ^ "\\not\\models " ^ (tex_of_id (-a)) ^ "\\\\\n"

    let rec tex_of_models_list = function
      []   -> ""
    | t::q -> (tex_of_models (fst t) (snd t)) ^ (tex_of_models_list q)

    let rec tex_of_models_not_list = function
      []   -> ""
    | t::q -> (tex_of_models_not (fst t) (snd t)) ^ (tex_of_models_not_list q)

    let tex_of_open_branch (pos, neg, gr) =
    begin
      constantes := [];
      index := 0;
      let nodes = List.map fst (fst gr) in
	"\\documentclass[10pt]{article}\n" ^
	"\\usepackage{amsmath,amssymb}\n" ^
	"\\usepackage{fullpage}\n" ^
	"\\newcommand{\\vp}[1]{{\mathrm #1}}\n" ^
        "\\newcommand{\\mtop}{\\vp{I}}\n" ^
        "\\newcommand{\\atop}{\\top}\n" ^
        "\\newcommand{\\abot}{\\bot}\n" ^
	"\\newcommand{\\mimp}{\\mathbin{-\\hspace{-0.1cm}\\ast}}\n" ^
	"\\newcommand{\\aimp}{\\rightarrow}\n" ^
	"\\newcommand{\\mand}{\\ast}\n" ^
	"\\newcommand{\\aand}{\\wedge}\n" ^
	"\\newcommand{\\aor}{\\vee}\n" ^
	"\\begin{document}\n" ^
	"\\begin{center}\n" ^
	"Countermodel for $" ^ (tex_of_formula !formule) ^ "$\n" ^
	"\\end{center}\n" ^
	"Worlds: $$ M = \\{" ^ (tex_of_label_list nodes) ^ "\\} $$\n" ^
	"Successors: $$ \\begin{array}{l}\n" ^ 
	(tex_of_graph_branch_list (fst gr)) ^ 
	"\\end{array} $$\n" ^
	"Forcing: $$ \\begin{array}{l}\n" ^ 
        (tex_of_models_list pos) ^ (tex_of_models_not_list neg) ^ 
	"\\end{array} $$\n" ^ 
	"\\end{document}"
   end

  end;;

