(*open Lexic;;*)
open Lexing;;
open Parsing;;

print_string "\nPropositional BI\nVersion 1.01\nCtrl+D to quit\n\n";;
flush stdout;;

let depth = ref 0;; 
let search_time = ref (0.);;
let search_result = ref "";;
let counter_fic = ref "";;
let counter_model = ref ([],[],([],[]));;

try
  let lexbuf = from_channel stdin in
    while true do
      begin
	print_string "BILL> "; 
	flush stdout;     
	let result = Syntax.main Lexic.token lexbuf in
          begin
            match result with
	    Prover.Types.Check p -> 
	      begin
		try
		  begin
		    search_result := "";
		    counter_model := ([],[],([],[]));
		    search_time := Unix.gettimeofday ();
		    Prover.Tableau.make_tableau !depth p;
		    search_time := (Unix.gettimeofday ()) -. !search_time;
		    search_result := "true";
		    print_string (!search_result ^ ": ");
		    print_string (Prover.BIHelpers.string_of_proposition p);
		    print_newline();
		  end
		with Prover.BIHelpers.Open_branch (pos, neg, gr) ->
		  begin
		    search_time := (Unix.gettimeofday ()) -. !search_time;
		    search_result := "false";
		    counter_model := (pos, neg, gr);
		    print_string (!search_result ^ ": ");
		    print_string (Prover.BIHelpers.string_of_proposition p);
		    print_newline();
		  end
		  | Prover.Types.Redundant_branch ->
		  begin
		    search_time := (Unix.gettimeofday ()) -. !search_time;
		    print_string "redundant branch\n";
		    print_string 
		      "sorry, no counter-model construction yet !!\n";
		    search_result := "false";
		    print_string (!search_result ^ ": ");
		    print_string (Prover.BIHelpers.string_of_proposition p);
		    print_newline();
		  end
	      end
	      | Prover.Types.Search_depth n -> 
		  begin 
		    depth := n;
		    print_string "search-depth set to ";
		    print_int n;
		    print_newline();
		  end
	      | Prover.Types.Stat ->
		  begin
                    print_string "search-time: ";
                    Printf.printf "%.3f" !search_time;
                    print_string "\nsearch-calls: ";
                    print_int !Prover.Tableau.count_call;
		    print_newline();
		  end
	      | Prover.Types.TeXcm s ->
		  if !search_result <> "false" then
		    print_string "!! no countermodel available\n"
		  else
		    begin
		      counter_fic := "tex/" ^ s ^ ".tex" ; 
		      try
			begin
			  let fic = open_out !counter_fic in
			    begin
			      output_string fic 
				(Prover.TexGen.tex_of_open_branch 
				   (!counter_model));
			      close_out fic;
			    end;
			    print_string 
			      ("countermodel written in " ^ (!counter_fic));
			    print_newline();
			end
		      with _ -> 
			begin
			  print_string ("cannot write file " ^ !counter_fic);
			  print_newline();
			end
		    end
	      | Prover.Types.Help -> print_string
		  ("Variables: [A-Za-uw-z]\n" ^
		   "Connectives: -*, *, ->, ^, v\n" ^
                   "Precedence: ->, -* < v < ^, *\n" ^
		   "Commands:\n  help: prints this page\n" ^
                   "  check <p>: decides proposition <p>\n" ^
                   "  depth <n>: sets search-depth to <n>\n" ^
		   "  stat: statistics about proof-search\n" ^
		   "  tex cm<file>: saves countermodel in a tex file\n" );
	      | Prover.Types.Syntax_error -> 
		  print_string "!! syntax error: ignoring input line\n"
	      | _ -> ()
          end
      end
    done
with Lexic.Eof -> exit 0 
  | Parse_error -> print_string "!! parse error\n"




