(*open Lexic;;*)
open Lexing;;
open Parsing;;

Prover.debug_mode := true;;

print_string "\nPropositional MILL\nVersion 1.01\nCtrl+D to quit\n\n";;
flush stdout;;

let search_time = ref (0.);;

try
  let lexbuf = from_channel stdin in
    while true do
      begin
	print_string "MILL> "; 
	flush stdout;     
	let result = Syntax.main Lexic.token lexbuf in
	  begin
            match result with
		Prover.Check p -> 
		  begin
		    Prover.count_call := 0;
		    search_time := Unix.gettimeofday ();
		    let res = Prover.Tableau.find_a_proof p in (
			search_time := Unix.gettimeofday () -. !search_time ; 
			if res then               
			  print_string "true: "
			else 
			  print_string "false: ";
			print_string (Prover.Mill.string_of_proposition p);
			print_newline();
		      )
		  end
              | Prover.Help -> print_string
		  ("Variables: [A-Za-z]\n" ^
		   "Connectives: -*, *\n" ^
		   "Precedence:  -* < *\n" ^
		   "Commands:\n  help: prints this page\n" ^
		   "  check <p>: decides proposition <p>\n" ^
		   "  stat: statistics about proof-search\n" );
	      | Prover.Stat ->
		  begin
                    print_string "search-time: ";
                    Printf.printf "%.3f" !search_time;
                    print_string "\nsearch-calls: ";
                    print_int !Prover.count_call;
		    print_newline();
		  end
	      | Prover.Syntax_error -> 
		  print_string "!! syntax error: ignoring input line\n"
	      | _ -> ()
	  end
      end
    done
with Lexic.Eof -> exit 0 
  | Parse_error -> print_string "!! parse error\n"




